# Create the database for the WebProject site.

# tables that are really used right now: (21)
#
# textfield language text msg translator
# project project_language projectcat projectcat_project project_org
# org orgcontact orgcontacttype
# user userpending user_org user_project
# forum message threadid
# session


# unused tables: (11)
#
# subproject task task_user taskdep taskstatus
# skill skillevel org_skill
# user_forum forummonitor security_log

# -------------------------------------------------------------------------
# Multi-Language text support

# This table is intended to be read by SELECT * FROM textfield
# For each (virtual) text column in a table, insert one row.  Note that there
# is no table_id, so the column name must be unique.
DROP TABLE IF EXISTS $$textfield;
CREATE TABLE $$textfield (
   field_id integer NOT NULL auto_increment,
   name varchar(40),
   PRIMARY KEY (field_id)
);

# List of available languages.
# text: language_name
DROP TABLE IF EXISTS $$language;
CREATE TABLE $$language (
   language_code char(2) NOT NULL,		# two-letter code
   language_id integer NOT NULL auto_increment,	# needed to access text
   PRIMARY KEY (language_code),
   INDEX (language_id)
);
INSERT INTO $$textfield(name) VALUES('language_name');
#INSERT INTO $$language(language_code) VALUES('en');
#INSERT INTO $$language(language_code) VALUES('de');
#INSERT INTO $$language(language_code) VALUES('es');
#INSERT INTO $$language(language_code) VALUES('fr');

# Here is the text table; all other tables don't contain text (unless it
# doesn't make sense to translate it).  Through(field_id,object_id) it is
# unambiguous to which object a text belongs.  (object_id is the primary key
# of any table).
DROP TABLE IF EXISTS $$text;
CREATE TABLE $$text (
   text_id integer NOT NULL auto_increment,
   field_id integer NOT NULL,			# -> text_field
   object_id integer NOT NULL,			# -> various tables
   language_code char(2) NOT NULL,		# -> language
   txt text NOT NULL,
#   PRIMARY KEY(field_id,object_id,language_code)
   PRIMARY KEY(text_id)
);

# A table with all the textual messages used directly from the code.  They are
# accessed through the gm2() "get message v2" function.  To reduce database
# load, the messages are exported into .php files which can be "include"d on
# demand.
#
# Translations:
#	0 ... 99	this is an original, has this many translations.
#	100		this is a translation.
DROP TABLE IF EXISTS $$msg;
CREATE TABLE $$msg (
        message_id varchar(40) binary NOT NULL,		# binary=case sensitive
        language_code char(2) NOT NULL,
	translations tinyint NOT NULL,

	# the version of *this* message.
        version tinyint NOT NULL,

	# what it was translated from. empty/0 for originals.
	origlang char(2) NOT NULL,
	origversion tinyint NOT NULL,

        txt text NOT NULL,
        PRIMARY KEY (message_id,language_code)
);


# -------------------------------------------------------------------------
# Projects

# text: name, description, purpose
DROP TABLE IF EXISTS $$project;
CREATE TABLE $$project (
   project_id integer NOT NULL auto_increment,
   unix_project_name varchar(30) DEFAULT '' NOT NULL,
   homepage varchar(128) DEFAULT '' NOT NULL,
   ts integer NOT NULL,
   status char(1) NOT NULL,			# A=active, P=pending, H=holding, D=deleted
   is_public char(1) DEFAULT '0' NOT NULL,
   use_mail char(1) DEFAULT '1' NOT NULL,
   use_forum char(1) DEFAULT '1' NOT NULL,
   use_pm char(1) DEFAULT '1' NOT NULL,
   use_news char(1) DEFAULT '1' NOT NULL,
   use_download char(1) DEFAULT '1' NOT NULL,
   rand_hash varchar(50),			# md5 digest is 32 chars.
   PRIMARY KEY(project_id),
   INDEX $$project_status_idx(status),
   INDEX $$project_is_public_idx(is_public)
);
INSERT INTO $$textfield(name) VALUES ('project_name');
INSERT INTO $$textfield(name) VALUES ('project_description');
INSERT INTO $$textfield(name) VALUES ('project_purpose');

# What languages this project is available in
DROP TABLE IF EXISTS $$project_language;
CREATE TABLE $$project_language (
   project_id integer NOT NULL,			# -> project
   language_code char(2) NOT NULL,		# -> language
   PRIMARY KEY(project_id,language_code),
   INDEX $$lang_idx(language_code)
);

# What organisations are part of this project; this also includes the requests
# for membership.  If approved is 0, then this entry is not yet valid.
DROP TABLE IF EXISTS $$project_org;
CREATE TABLE $$project_org (
   project_id integer NOT NULL,			# -> project
   org_id integer NOT NULL,			# -> org
   approved char(1) DEFAULT '0' NOT NULL,
   ts integer NOT NULL,
   projectadmin_id integer NOT NULL,		# -> user
   orgadmin_id integer NOT NULL,		# -> user
   PRIMARY KEY(project_id,org_id)
);

# text: statusname
#CREATE TABLE projectstatus (
#   projectstatus_id integer NOT NULL auto_increment,
#   PRIMARY KEY (projectstatus_id)
#);

# -------------------------------------------------------------------------
# Categorisation of the projects

# text: shortname, name, description
DROP TABLE IF EXISTS $$projectcat;
CREATE TABLE $$projectcat (
   projectcat_id integer NOT NULL auto_increment,
   parent_id integer NOT NULL,			# -> projectcat.  0=toplevel
   root_parent_id integer NOT NULL,		# -> projectcat. may be == projectcat_id
   subcat_count integer DEFAULT '0' NOT NULL,	# number of immediate sub categories
   project_count integer DEFAULT '0' NOT NULL,	# projects in this category
   subproject_count integer DEFAULT '0' NOT NULL, # projects in all subcategories
   path varchar(255) DEFAULT '' NOT NULL,
   PRIMARY KEY (projectcat_id),
   INDEX $$projectcat_parent_idx (parent_id),
   INDEX $$projectcat_root_parent_idx (root_parent_id)
);
INSERT INTO $$textfield(name) VALUES ('projectcat_shortname');
INSERT INTO $$textfield(name) VALUES ('projectcat_name');
INSERT INTO $$textfield(name) VALUES ('projectcat_description');

#CREATE TABLE projectcatsum (
#	projectcat_id integer NOT NULL,		# -> projectcat
#	subprojects integer NOT NULL,
#	PRIMARY KEY (projectcat_id)
#);

# This defines which projects are in which categories.
DROP TABLE IF EXISTS $$projectcat_project;
CREATE TABLE $$projectcat_project (
   project_id integer NOT NULL,			# -> project
   projectcat_id integer NOT NULL,		# -> projectcat
   projectcat_root_id integer NOT NULL,		# -> projectcat
   PRIMARY KEY (project_id,projectcat_id),
   INDEX $$g_g_projectcat_idx (projectcat_id)
);

# -------------------------------------------------------------------------
# Projects & Tasks

# text: name
DROP TABLE IF EXISTS $$subproject;
CREATE TABLE $$subproject (
   subproject_id integer NOT NULL auto_increment,
   project_id integer NOT NULL,			# -> project
   is_public tinyint NOT NULL,
   PRIMARY KEY (subproject_id),
   INDEX $$subproject_project_idx (subproject_id)
);
INSERT INTO $$textfield(name) VALUES ('subproject_name');

# text: summary, details
DROP TABLE IF EXISTS $$task;
CREATE TABLE $$task (
   task_id integer NOT NULL auto_increment,
   subproject_id integer NOT NULL,		# -> subproject
   percent_complete tinyint unsigned DEFAULT '0' NOT NULL,
   priority tinyint unsigned DEFAULT '0' NOT NULL,
   hours float(10,2) DEFAULT '0.00' NOT NULL,
   start_date integer NOT NULL,
   end_date integer NOT NULL,
   created_by integer NOT NULL,			# -> user
   status_id integer NOT NULL,			# -> taskstatus
   PRIMARY KEY (task_id),
   INDEX $$task_subproject_idx (subproject_id)
);
INSERT INTO $$textfield(name) VALUES ('task_summary');
INSERT INTO $$textfield(name) VALUES ('task_details');

# assignment
DROP TABLE IF EXISTS $$task_user;
CREATE TABLE $$task_user (
   task_id integer NOT NULL,			# -> task
   user_id integer NOT NULL,			# -> user
   PRIMARY KEY (task_id,user_id),
   INDEX $$t_u_task_idx (task_id),
   INDEX $$t_u_user_idx (user_id)
);

# dependencies between tasks: task1 can be started only after task2 has been
# completed.
DROP TABLE IF EXISTS $$taskdep;
CREATE TABLE $$taskdep (
   taskdep_id integer NOT NULL auto_increment,
   task1_id integer NOT NULL,			# -> task
   task2_id integer NOT NULL,			# -> task
   PRIMARY KEY (taskdep_id),
   INDEX $$taskdep_task1 (task1_id),
   INDEX $$taskdep_task2 (task2_id)
);

# text: name
DROP TABLE IF EXISTS $$taskstatus;
CREATE TABLE $$taskstatus (
   taskstatus_id integer NOT NULL auto_increment,
   PRIMARY KEY (taskstatus_id)
);
INSERT INTO $$textfield(name) VALUES('taskstatus_name');

# -------------------------------------------------------------------------
# Organi[zs]ations

# text: name, description
DROP TABLE IF EXISTS $$org;
CREATE TABLE $$org (
   org_id integer NOT NULL auto_increment,
   is_centre tinyint NOT NULL,
   ts integer NOT NULL,
   status char(1) NOT NULL,			# A=active D=deleted P=pending
   PRIMARY KEY (org_id)
);
INSERT INTO $$textfield(name) VALUES('org_name');
INSERT INTO $$textfield(name) VALUES('org_description');

DROP TABLE IF EXISTS $$orgcontact;
CREATE TABLE $$orgcontact (
   orgcontact_id integer NOT NULL auto_increment,
   org_id integer NOT NULL,			# -> org
   contacttype_id integer NOT NULL,		# -> orgcontacttype
   value text,
   PRIMARY KEY (orgcontact_id),
   INDEX $$orgcontact_org_idx (org_id)
);

# text: orgcontacttype_name
# types of contact: email homepage fax tel addr
DROP TABLE IF EXISTS $$orgcontacttype;
CREATE TABLE $$orgcontacttype (
   orgcontacttype_id integer NOT NULL auto_increment,
   PRIMARY KEY (orgcontacttype_id)
);
INSERT INTO $$textfield(name) VALUES('orgcontacttype_name');


# text: name
DROP TABLE IF EXISTS $$skill;
CREATE TABLE $$skill (
   skill_id integer NOT NULL auto_increment,
   PRIMARY KEY (skill_id)
);
INSERT INTO $$textfield(name) VALUES('skill_name');

# text: name
DROP TABLE IF EXISTS $$skillevel;
CREATE TABLE $$skillevel (
   skillevel_id integer NOT NULL auto_increment,
   seq integer NOT NULL,			# specifies the sort order
   PRIMARY KEY (skillevel_id)
);
INSERT INTO $$textfield(name) VALUES('skillevel_name');

DROP TABLE IF EXISTS $$org_skill;
CREATE TABLE $$org_skill (
   org_skill_id integer NOT NULL auto_increment,
   org_id integer NOT NULL,			# -> org
   skill_id integer NOT NULL,			# -> skill
   skillevel_id integer NOT NULL,		# -> skillevel
   PRIMARY KEY (org_skill_id),
   INDEX $$o_s_org_idx (org_id)
);


# -------------------------------------------------------------------------
# Users

DROP TABLE IF EXISTS $$user;
CREATE TABLE $$user (
   user_id integer NOT NULL auto_increment,
   user_name varchar(20) NOT NULL,
   fullname varchar(40) NOT NULL,
   country_id integer NOT NULL,			# -> country
   email varchar(50) NOT NULL,
   password varchar(32) NOT NULL,
   status char(1) NOT NULL,			# A=active, D=deleted, P=pending, L=locked

   p_siteadmin char(1) DEFAULT '0' NOT NULL,	# is site manager
   p_projectmanage char(1) DEFAULT '0' NOT NULL,	# approve/del projects
   p_orgmanage char(1) DEFAULT '0' NOT NULL,	# approve/del orgs
   p_translate char(1) DEFAULT '0' NOT NULL,	# make translations?
   p_useradmin char(1) DEFAULT '0' NOT NULL,	# can change users?

   browser_mode char(1) DEFAULT 'N' NOT NULL,	# N=normal F=frames T=textonly S=simple
   mail_siteupdates char(1) DEFAULT 'Y' NOT NULL,	# Y or ''
   add_date integer NOT NULL,
   languages varchar(30) NOT NULL,		# list of IDs joined by ` '
   vars text NOT NULL,				# "var=value" pairs
   PRIMARY KEY (user_id),
   INDEX $$user_status_idx (status)
);

# This table contains data an entry per pending user.
# When a user registers, we'll add a "user" entry as well as a "user-pending"
# entry.  Then we send a mail to the address he specified, including
# instructions how to activate his account.  A "confirm-hash" is included
# which is needed.  After a given time this times out and the
# user & user-pending entries are deleted.  This means that the user-pending
# table has to be scanned regularly, which should be faster than scanning the
# whole user table.
DROP TABLE IF EXISTS $$userpending;
CREATE TABLE $$userpending (
   confirm_hash varchar(32) NOT NULL,
   user_id integer NOT NULL,			# -> user
   add_date integer NOT NULL,			# to expire entries
   PRIMARY KEY (confirm_hash),
   INDEX $$userpending_user_idx (user_id)
);

# Affiliation of users with organisations.
DROP TABLE IF EXISTS $$user_org;
CREATE TABLE $$user_org (
   org_id integer NOT NULL,
   user_id integer NOT NULL,
   ts integer NOT NULL,
   p_admin char(1) DEFAULT '0' NOT NULL,	# is admin, i.e. can't be removed
   p_adduser char(1) DEFAULT '0' NOT NULL,	# can add new members
   p_deluser char(1) DEFAULT '0' NOT NULL,	# can remove non-admins
   p_chfn char(1) DEFAULT '0' NOT NULL,		# can change description etc.
   p_chmod char(1) DEFAULT '0' NOT NULL,	# can change p_xx of users but not of admins
   PRIMARY KEY (org_id,user_id)
);


# Project membership.
DROP TABLE IF EXISTS $$user_project;
CREATE TABLE $$user_project (
   user_id integer NOT NULL,			# -> user
   project_id integer NOT NULL,			# -> project
   ts integer NOT NULL,				# member since when
   p_admin char(1) DEFAULT '0' NOT NULL,	# admin: can't be removed
   p_adduser char(1) DEFAULT '0' NOT NULL,	# can add new members
   p_deluser char(1) DEFAULT '0' NOT NULL,	# can kick members out
   p_chfn char(1) DEFAULT '0' NOT NULL,		# can change name/description
   p_chmod char(1) DEFAULT '0' NOT NULL,	# can change perms (but not of admins unless also admin)
   p_forum_read char(1) DEFAULT '1' NOT NULL,	# can read forums?
   p_forum_post char(1) DEFAULT '1' NOT NULL,	# can post in forums?

   p_forum char(1) DEFAULT '0' NOT NULL,	# 0=on, 1=tech, 2=both, 3=admin
   PRIMARY KEY (user_id,project_id),
   INDEX (project_id)
);


# what translations can the user do?
DROP TABLE IF EXISTS $$translator;
CREATE TABLE $$translator (
	user_id integer NOT NULL,
	lang_from char(2) NOT NULL,
	lang_to char(2) NOT NULL,
	PRIMARY KEY (user_id,lang_from,lang_to)
);


# -------------------------------------------------------------------------
# Messaging

# A forum has a topic, like "Help", "Suggestions", etc.  Allowed languages:
# either unchecked, or limited by the languages the forum name is available
# in.
# text: name
DROP TABLE IF EXISTS $$forum;
CREATE TABLE $$forum (
   forum_id integer NOT NULL auto_increment,
   project_id integer NOT NULL,			# -> project
   p_read char(1) DEFAULT '1' NOT NULL,		# users can read?
   p_post char(1) DEFAULT '1' NOT NULL,		# users can post?
   PRIMARY KEY (forum_id),
   INDEX $$forum_project_idx (project_id)
);
INSERT INTO $$textfield(name) VALUES('forum_name');

# individual permissions UNUSED
DROP TABLE IF EXISTS $$user_forum;
CREATE TABLE $$user_forum (
   user_id integer NOT NULL,			# -> user
   forum_id integer NOT NULL,			# -> forum
   p_read char(1) DEFAULT '0' NOT NULL,		# can read this forum?
   p_post char(1) DEFAULT '0' NOT NULL,		# can post in this forum?
   PRIMARY KEY (user_id,forum_id)
);

# An individual message.  Every message belongs to a thread; posting a new
# message automatically start a thread, followups stay in the same thread.
# Note: all messages of one thread should be in the same language.
# Starts of threads are all messages whose "followup_to" is 0; their subject
# also defines the name of the thread.
DROP TABLE IF EXISTS $$message;
CREATE TABLE $$message (
   message_id integer NOT NULL auto_increment,
   forum_id integer NOT NULL,			# -> forum
   posted_by integer NOT NULL,			# -> user
   followup_to integer NOT NULL,		# -> message
   thread_id integer NOT NULL,
   ts integer NOT NULL,
   language_code char(2) NOT NULL,		# which language
   subject varchar(255) NOT NULL,
   body text NOT NULL,
   PRIMARY KEY (message_id),
   INDEX(thread_id), INDEX(forum_id)
);

# this table is used to generate unique thread IDs.  It should be cleaned out
# once in a while (just keep the highest numer).
DROP TABLE IF EXISTS $$threadid;
CREATE TABLE $$threadid (
	thread_id integer DEFAULT '1' NOT NULL auto_increment,
	PRIMARY KEY(thread_id)
);

# This allows users to be notified when messages are posted in specified
# forums.
DROP TABLE IF EXISTS $$forummonitor;
CREATE TABLE $$forummonitor (
   forummonitor_id integer NOT NULL auto_increment,
   forum_id integer NOT NULL,			# -> forum
   user_id integer NOT NULL,			# -> user
   PRIMARY KEY (forummonitor_id),
   INDEX $$forummonitor_forum_idx (forum_id),
   INDEX $$forummonitor_user_idx (user_id)
);

# -------------------------------------------------------------------------
# Administration

# Each user currently logged in has one entry in this table.  Through the
# cookie set (session_hash), this table is queried every time the user sends a
# request.
# when_last: When the last access was.  This is updated max. once per minute,
# and whenever the session has to be written to store session variables.
DROP TABLE IF EXISTS $$session;
CREATE TABLE $$session (
   session_hash char(32) NOT NULL,
#  ts integer NOT NULL,				# to detect dead sessions
   when_start integer NOT NULL,			# when the session was created
   when_last integer NOT NULL,			# See note.
   ip_addr varchar(25) NOT NULL,		# allow for ipv6 addresses.
   user_id integer NOT NULL,			# -> user
   vars text NOT NULL,				# "var=value" pairs
   PRIMARY KEY (session_hash),
   INDEX $$session_user_idx (user_id),		# new session -> delete all other sessions of this user.
   INDEX $$session_when_idx (when_last)		# quick scan for timed-out sessions.
);

# Some noteworthy event occurred.  This can be changing of passwords, etc.
DROP TABLE IF EXISTS $$security_log;
CREATE TABLE $$security_log (
   security_log_id integer NOT NULL auto_increment,
   user_id integer NOT NULL,			# -> user
   ts integer NOT NULL,
   category varchar(15) NOT NULL,
   description varchar(255),
   PRIMARY KEY (security_log_id)
);

