<?php
# Export all the messages from the database and merge with possibly existing
# message files (the editable ones named <modulename>.<languagecode>
# These files can be read back again by langimport.php.
# $Id: langexport.php,v 1.4 2001/11/11 22:23:21 sf Exp $

/*

- select which modules (like langimport)
- search for modules, build list (like langimport)

FOR EACH MODULE:
	- get all messages from the DB (NEW)

FOR EACH LANGUAGE FILE:
	- read the file, split into individual messages (like langimport)

FOR EACH MESSAGE:
	- if it's not in the DB, take the file's version.
	- compare versions of file version and DB version.  Take the newer
	  one; if same, compare text; if different, then what??  Maybe put
	  into the output file TWICE.
	- store in a memory table.
END for each message.

	- add all messages in the current language from the DB that haven't
	  been found in the file to the output table.
	- sort the messages by message name.
	- make a backup of the output file.
	- write the output file.

END for each language

	- if any DB entries are left over, then they are in a language that
	  didn't yet exist as a file.  Write (sorted).

END for each module

*/

include '../include/pre.php';
include 'langtool.php';

$odir = "$BASEDIR$tr_dir";
$origlang_default = 'en';		// should not be needed.
$just_kidding = 0;			// don't really write.

if( !isset($BuildButton) ) {
	site_header('langexport');
?><br>

<h2> Welcome to the language file Exporter.</h2>

<p>
You can <b>export</b> translation files <b>from the database</b>.
Please specify the module ID to be imported (as a LIKE expression).
No entry means <i>all</i> modules.
</p>

<p>
Note: the existing files will be read first, so if they contain any newer
versions of messages or any messages that are not in the database, nothing
will be lost.  Backups are made anyway.
</p>

<form action="langexport.php" method="post">
<input type=text name="form_fname" size=40>
<input type="submit" name="BuildButton" value="Build">
</form>
<?php
	site_footer();
	exit;
}


// ----------------- functions ------------------

// This one message has been read from the file.  Check for validity, and then
// enter it into the $messages array.
function process_message($msgid,$version,&$text,$lang,$origlang,$modid,$origversion) {
	global $messages, $file_changed, $msgids, $count;

	if( !$msgid && !$text )
		return;

	if( !$msgid ) {
		print "<b>Error</b> Missing msgid.<br>\n";
		return;
	}
	if( !$text ) {
		print "<b>Error</b> Missing text for $msgid.<br>\n";
		return;
	}

	// beautify text, i.e. remove newlines and spaces at the beginning and
	// end.
	trim_text($text);

	// was this message also in the database?
	if( !isset($messages[$lang][$msgid]) ) {
		// no. Just add - make an array like db_fetch_array().
		$messages[$lang][$msgid] = array(
			'txt' => $text,
			'version' => $version,
			'origversion' => $origversion,
			'origlang' => $origlang,
			'language_code' => $lang
		);
		print "Just in the file: $msgid<br>\n";
		$count['msgs-from-file']++;
		return;
	}

	// message already exists...  Which one should we use?
	$msg = $messages[$lang][$msgid];
	// mark the message as seen.
	unset($msgids[$msgid]);
	
	// message in the DB is newer?
	if( $msg['version'] > $version ) {
		$file_changed ++;
		$count['msgs-from-db']++;
		return;
	}

	// message in the DB is older?
	if( $msg['version'] < $version ) {
		$messages[$lang][$msgid] = array(
			'txt' => $text,
			'version' => $version,
			'origversion' => $origversion,
			'origlang' => $origlang,
			'language_code' => $lang
		);
		print "Version in file is newer: $msgid<br>\n";
		$count['msgs-from-file']++;
		return;
	}

	// same version...
	if( $msg['txt'] != $text )
		trim_text($msg['txt']);

	if( $msg['txt'] == $text ) {
		// original: don't check origlang and origversion.
		if( $msg['translations'] < 100 ) {
			$count['msgs-same'] ++;
			return;
		}

		// but still, origversion or origlang could differ...  When
		// this happens, the version should have increased too, but
		// maybe the translator forgot that.
		if( $msg['origlang'] != $origlang ) {
			print "$msgid: Original language differs: file=$origlang, DB=$msg[origlang]<br>\n";
		}
		if( $msg['origversion'] != $origversion ) {
			print "$msgid: Original version differs: file=$origversion, DB=$msg[origversion]<br>\n";
			if( $msg['origversion'] < $origversion ) {
				$msg['origversion'] = $origversion;
				$file_changed = 1;
			}
		}

		$count['msgs-same'] ++;
		return;
	}

	// same version, different text.  This *should* not happen.
	print "$msgid: same version, different text.  Using the DB version.<br>\n";
	print " &nbsp; DB: $msg[txt]<br>\n";
	print " &nbsp; FILE: $text<br>\n";
	$count['collisions'] ++;
	$file_changed ++;
}

?><?php

// Read one translation file, merge with the database's messages already in
// memory.  Returns 1 if the DB contains newer information than the file, i.e.
// the file must be written.
function process_file($modid,$lang) {

	global $msgids, $file_changed, $messages;
	global $origlang_default, $count;

	$fname = $modid . '.' . $lang;
	if( !file_exists($fname) ) {
		print " - <b>doesn't exist.</b>\n";
		return 1;
	}

	$fd = fopen( $fname, 'r' );
	if( !$fd ) {
		print " - <b>can't open</b>\n";
		return 1;
	}
	$count['files-read'] ++;

	// build a list of all IDs in the DB, so we can remove them when a
	// matching message is in the file.  The left-overs are the messages
	// only in the DB.
	$msgids = array();
	reset($messages[$lang]);
	while(list($k,$v)=each($messages[$lang]))
		$msgids[$k] = 1;

	$msgid = '';
	$text = '';
	$version = '';
	$file_changed = 0;
	$origlang = $origlang_default;
	$origversion = 1;

	while( $line = fgets($fd,500) ) {

		// comment?
		if( substr($line,0,1) == '#' )
			continue;

		// start of new message?
		if( substr($line,0,1) == '*' ) {
			process_message($msgid,$version,$text,$lang,$origlang,$modid,$origversion);
			$text = '';
			list($msgid,$version,$origlang,$origversion)
				= parse_msgid($line,$origlang_default);
			continue;
		}

		// before the start of the first message?
		if( $msgid == '' )
			continue;

		// reading a message - append to buffer.
		$text .= $line;
	}
	fclose( $fd );

	// Flush the last message.  Same code as above.
	process_message($msgid,$version,$text,$lang,$origlang,$modid,$origversion);

	// did we find any newer version of messages?
	if( $file_changed )
		return 1;

	// There is still the possibility that the DB contains messages not in
	// the file.
	if( count($msgids) ) {
		reset($msgids);
		while(list($k,$v)=each($msgids))
			print "leftover: $k<br>\n";
		return 1;
	}

	// no need to write the file.
	return 0;
}

?><?php

// now write the file.
function file_write($modid,$lang) {
	global $messages, $count, $just_kidding;
	print "need to write $modid.$lang<br>\n";

	// first, make a backup of the old file (if it exists).
	$ofname = "$modid.$lang";

	if( !$just_kidding ) {
		if( file_exists($ofname) ) {
			if( file_exists("$ofname.bak") )
				unlink("$ofname.bak");
			if( !rename($ofname,"$ofname.bak") ) {
				print "<b>ERROR: could not rename $ofname...</b><br>\n";
				return;
			}
		}

		// now write it.
		$of = fopen( $ofname, 'w' );
		if( !$of ) {
			print "<B>Can't open output file $ofname!</B>\n";
			return;
		}
	}

	$count['files-written'] ++;

	// format of a message in the file:
	// *msgid version origlang origversion
	ksort($messages[$lang]);
	reset($messages[$lang]);
	while(list($msgid,$row)=each($messages[$lang])) {
		$v = '';
		if( $row['version'] != 1 )
			$v = " $row[version]";
		if( $row['origlang'] )
			$v .= " $row[origlang]";
		if( $row['origversion'] > 1 )
			$v .= " $row[origversion]";
		if( $just_kidding )
			print "*$msgid$v<br>\n";
		else
			fputs($of,"*$msgid$v\n$row[txt]\n\n");
	}

	if( !$just_kidding )
		fclose($of);
}

?><?php

// Process one module.
// msg_list: list of messages in the DB.
// file_list: list of language codes where a file is available.
function process_module($modid,$lang_list) {
	global $messages, $count, $db_prefix;

	print $modid . "\n";
	
	// first, get all messages that are in the DB for this module
	// in all languages.
	$messages = array();
	$res = db_query("SELECT * FROM $db_prefix"."msg "
		. "WHERE message_id LIKE '$modid:%'");
	if(!$res) {
		print db_query_error();
		return;
	}

	$count['select'] ++;
	while( $row = db_fetch_array($res) ) {
		$count['fetch']++;
		list($x,$msgname) = split(':',$row['message_id'],2);
		if( $x != $modid ) {
			print "<b>error: got moduleid $x in module $modid...<b><br>\n";
			continue;
		}
		$messages[$row['language_code']][$msgname] = $row;
	}
	unset($res);

	// for each language, try to read the file and  and update the
	// "messages" array with whatever is newer.  Then write.
	print "<ul>\n";
	reset($lang_list);
	while( list($lang,$dummy) = each($lang_list) ) {
		// read the file and merge
		print "<li>$lang\n";
		$file_changed = process_file($modid,$lang);
		if( $file_changed )
			file_write($modid,$lang);
		unset($messages[$lang]);
	}

	// write any $messages that might still be left: when the file for a
	// language didn't exist yet.
	reset($messages);
	while(list($lang,$msgs)=each($messages)) {
		print "<li>NEW: $lang\n";
		file_write($modid,$lang);
	}

	print "</ul>\n";
}



?><?php
// --------------------- main ----------------------

site_header('langbuildresult');
$count = array();
$count['select'] = $count['fetch'] = $count['files-read'] = 0;
$count['msgs-same'] = 0;
$count['msgs-from-db'] = 0;
$count['msgs-from-file'] = $count['collisions'] = 0;
$count['files-written'] = 0;

// Don't do this before site_header, because it uses relative paths...
$old_dir = getcwd();
if (!chdir($odir))
	exit_error('ERROR',"<H1>Can't chdir to $odir.</H1>\n");

// find all matching files.
$d = opendir(".");
if (!$d)
	exit_error("ERROR","Can't read directory $odir.");

?><?php


// find out which modules from the DB to process, and in which languages they
// are available.
$sql = "SELECT message_id,language_code FROM ".$db_prefix."msg";
if( isset($form_fname) && $form_fname ) {
	$sql .= " WHERE message_id LIKE '$form_fname:%'";
}

$res = db_query( $sql );
if( !$res ) print db_query_error();
$count['select']++;
$count_msgs = 0;
$modules_db=array();
while( $row = db_fetch_array($res) ) {
	$count['fetch'] ++;
	list($modid,$msgname) = split(':',$row['message_id'],2);
	$modules_db[$modid][$row['language_code']] = 1;
	$count_msgs ++;
}
print "Found $count_msgs individual messages for ".count($modules_db)." modules to process.<br>";

// we now have a list of message files and a list of messages in the DB.
umask(002);	// create files with project write permission.

// process each module.
reset($modules_db);
while( list($modid,$v) = each($modules_db) ) {
	process_module($modid,$v);
}

// show statistics
print "<hr><h2>Statistics</h2>\n<ul>\n";
reset($count);
while(list($k,$v)=each($count)) {
	if( $v )
		print "<li>$k: $v\n";
}
print "</ul>\n";

// The site footer will display an image, for this try to get the image's
// size, and for this use a relative path...  Better change the directory back
// to its previous value.
chdir($old_dir);
site_footer();
?>
