<?php
# Check validity of passwords, usernames, projectnames etc.
# $Id: account.php,v 1.3 2001/11/10 10:05:17 sf Exp $

# Check for a valid password.  Returns TRUE if valid.
function account_pwvalid($pw) {
	global $register_error;
        if (strlen($pw) < 6) {
                $register_error = gm('account:min6');
                return 0;
        }
        return 1;
}


# Check for a valid username (before registering a user).
# Returns TRUE if the username is good.
function account_namevalid($name) {

	global $register_error;

	// no spaces
	if (strrpos($name,' ') > 0) {
		$register_error = gm('account:nospaces');
		return 0;
	}

	// must have at least one character
	if (strspn($name,"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ") == 0) {
		$register_error = "There must be at least one character.";
		return 0;
	}

	// must contain all legal characters
	//if (strspn($name,"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!@#\$%^&*()-_\\/{}[]<>+=|;:?.,`~")
	if (strspn($name,"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_")
		!= strlen($name)) {
		$register_error = gmx('account:illegalchar');
		return 0;
	}

	// min and max length
	if (strlen($name) < 3) {
		$register_error = gm('account:tooshort');
		return 0;
	}
	if (strlen($name) > 15) {
		$register_error = gm('account:toolong');
		return 0;
	}

	// illegal names
	if (eregi("^((root)|(bin)|(daemon)|(adm)|(lp)|(sync)|(shutdown)|(halt)|(mail)|(news)"
		. "|(uucp)|(operator)|(games)|(mysql)|(httpd)|(nobody)|(dummy)"
		. "|(www)|(cvs)|(shell)|(ftp)|(irc)|(debian)|(ns)|(download))$",$name)) {
		$register_error = gm('account:reserved');
		return 0;
	}
	if (eregi("^(anoncvs_)",$name)) {
		$register_error = gm('account:reserved');
		return 0;
	}
		
        return 1;
}

function account_projectnamevalid($name) {
	if (!account_namevalid($name)) return 0;
	global $register_error;
	
	// illegal names
	if (eregi("^((www[0-9]?)|(cvs[0-9]?)|(shell[0-9]?)|(ftp[0-9]?)|(irc[0-9]?)|(news[0-9]?)"
		. "|(mail[0-9]?)|(ns[0-9]?)|(download[0-9]?)|(pub)|(users)|(compile)|(lists)"
		. "|(slayer)|(orbital)|(tokyojoe)|(webdev)|(projects)|(cvs)|(slayer)|(monitor)|(mirrors?))$",$name)) {
		$register_error = gm('account:reserved'); // "Name is reserved for DNS purposes.";
		return 0;
	}

	if (eregi("_",$name)) {
		$register_error = gm('account:underscore');
		return 0;
	}

	return 1;
}

// The following is a random salt generator
function account_gensalt() {
	function rannum(){             
		mt_srand((double)microtime()*1000000);                  
		$num = mt_rand(46,122);                  
		return $num;                  
	}             
	function genchr(){
		do {          
			$num = rannum();                  
		} while ( ( $num > 57 && $num < 65 ) || ( $num > 90 && $num < 97 ) );          
		$char = chr($num);          
		return $char;          
	}           

	$a = genchr(); 
	$b = genchr();
	$salt = "$1$" . "$a$b";
	return $salt;        
}

// generate unix pw
function account_genunixpw($plainpw) {
	return crypt($plainpw,account_gensalt());
}

/*
// returns next userid
function account_nextuid() {
	global $db_prefix;
	$res = db_query("SELECT max(unix_uid) AS maxid FROM ".$db_prefix."user");
	$row = db_fetch_array($res);
	return ($row[maxid] + 1);
}
*/

/*
// print out shell selects
function account_shellselects($current) {
	$shells = file("/etc/shells");

	for ($i = 0; $i < count($shells); $i++) {
		$this_shell = chop($shells[$i]);

		if ($current == $this_shell) {
			echo "<option selected value=$this_shell>$this_shell</option>\n";
		} else {
			echo "<option value=$this_shell>$this_shell</option>\n";
		}
	}
}
*/

?>
