<?php

/* Authentication plug-ins: called by nav_check(), which is driven by
 * $SITEMAP.
 *
 * The functions are all named auth_xxx, the xxx being anything that can be
 * used as a constraint in $SITEMAP, like "superuser". Parameters to the
 * functions:
 *	$v	the parameter given in $SITEMAP (the "value" part of the
 *		key/value pair)
 *	$mode	What we're just doing: 0=authentication  1=building nav_path
 *		2=building nav_next.
 *
 *  Return codes expected from the auth_xxx functions:
 *	0=permission granted
 *	1=denied $WHYNOT is the message ID of an explanation.
 *	2=as 1, but the cause is related to the user not being logged in.
 *	3=some prerequisite failed: missing or invalid parameter, db error
 *
 * $Id: auth.php,v 1.3 2001/11/10 10:07:47 sf Exp $
 */

// must be superuser.
function auth_superuser() {
	global $WHYNOT, $USER;

	if( isset($USER) && $USER['p_siteadmin'] )
		return 0;
	$WHYNOT='nav:superuser';
	return 2;
}

/* Sometimes, a page shouldn't be listed in the subpages section (nav_next),
 * like the generic "help" page which can show up as subpage of any page
 * (using $NAVPATH), but technically is a subpage of Root.  Don't show it
 * there.  Same goes for the error page etc.
 */
function auth_dontshow($v, $mode) {
	// error_log("dontshow v=$v, mode=$mode");
	return ($mode == 2) ? 1 : 0;
}

// user must have a certain permission
function auth_user($v) {
	global $WHYNOT, $USER;
	if( isset($USER) && $USER["p_$v"] )
		return 0;
	$WHYNOT="nav:userperm_$v";	
	return 2;
}

// Require $ORG[status] to be found in $v.
function auth_org_status($v) {
	global $WHYNOT, $ORG;
	if( strpos("_$v",$ORG['status']) )
		return 0;
	$WHYNOT="nav:org_$ORG[status]";
	return 1;
}

function auth_org_perm($v) {
	global $WHYNOT, $org_id;
	if( user_org_perm($v) ) return 0;
	$WHYNOT='nav:orgperm';
	return 1;
}

// org_id must be set.	For non-active projects, must be org manager.
// $v:	'': no additional constraints
//	'm': must be member
//	else: user_org_perm($v) is called.
function auth_org_id($v,$mode) {
	global $WHYNOT, $linkadd, $org_id, $ORG, $db_prefix;

	if( !isset($ORG) ) {
		if( !isset($org_id) || !$org_id ) {
			$WHYNOT='nav:noorg';
			return 3;
		}
		$res = db_query( "SELECT * FROM ".$db_prefix."org "
			. "WHERE org_id=$org_id" );
		if( !$res ) {
			$WHYNOT='nav:dberror';
			return 3;
		}
		if( db_numrows($res) < 1 ) {
			$WHYNOT='nav:invorg';
			return 3;
		}
		$ORG = db_fetch_array($res);

		$linkadd = "?org_id=$org_id";
		return 0;
	}

	// org_id is valid.
	if( $ORG['status'] != 'A' && !user_perm('orgmanage') ) {
		$WHYNOT='nav:orgaccess';
		return 3;
	}

	$linkadd = "?org_id=$org_id";
	return 0;
}

?><?php
// project: '': project_id must be valid
// 'm' ... must be member
// 'A' ... must be project admin
// 'f' ... must be forum tech
// 'F' ... must be forum admin
function auth_project($p) {
	global $WHYNOT, $project_id, $PROJECT, $USER, $linkadd;
	global $auth_project, $db_prefix;

	// set by forum to enforce project membership for private forums
	if( !$p && isset($auth_project) )
		$p = $auth_project;

	if( !isset($project_id) || !$project_id ) {
		$WHYNOT='nav:noproject';
		return 3;
	}

	// for checking navigation, this might be called more than once...
	if( !isset($PROJECT) ) {
		$res = db_query("SELECT * FROM ".$db_prefix."project "
			. "WHERE project_id=$project_id");
		if( !$res ) {
			$WHYNOT='nav:dberror';
			$project_id=0;
			return 3;
		}
		if (db_numrows($res) < 1) {
			$WHYNOT='nav:invproject';
			$project_id=0;
			return 3;
		}
		$PROJECT = db_fetch_array($res);
	}

	// if project is private, must be member
	if( !$PROJECT['is_public'] && $p == '' )
		$p = 'm';

	// if project is pending, and user is site admin -> permitted
	if( $PROJECT['status'] == 'P' && isset($USER) && $USER['p_projectmanage'] && ($p=='m'||$p=='A') ) {
		$p = '';

	// otherwise, non-active projects can't be viewed.
	} elseif( $PROJECT['status'] != 'A' ) {
		$WHYNOT="nav:project_$PROJECT[status]";
		return 1;
	}

	// do this before exiting with code 2, because if disable_auth
	// is set, the link will still show up.
	if( !isset($linkadd) || !$linkadd )
		$linkadd = "?project_id=$PROJECT[project_id]";

	if( $p && !user_project_perm($p) ) {
		$WHYNOT="nav:project_$p";
		return 2;
	}

	return 0;
}

// can the current user edit the user $uid ?
function auth_projectedituser($u) {
	global $WHYNOT, $uid, $USER;

	if( !$u ) {
		if( !isset($uid) || !$uid ) {
			$WHYNOT='nav:nouser';
			return 1;
		}
		$u = $uid;
	}

//	error_log("auth_projectedituser: validating user $u");

	// the user must be member
	if( !user_project_perm('',$u) ) {
		$WHYNOT='nav:invuser';
		return 1;
	}

	// can edit self (but only if you're member...)
	if( $USER['user_id'] == $u ) return 0;

	// if admin of this project, OK
	if( user_project_perm('admin') ) return 0;

	// if general project admin, OK
	if( user_perm('projectmanage') ) return 0;

	// if allowed to chmod, and uid is *not* admin, OK
	if( user_project_perm('chmod') && !user_project_perm('admin',$u) ) return 0;

	$WHYNOT='nav:cantedit';
	return 1;
}

// msg_id must be valid.
function auth_message() {
	global $WHYNOT, $linkadd, $msg_id;
	global $MESSAGE, $forum_id, $thread_id, $db_prefix;
	
	if( !isset($msg_id) || !$msg_id ) {
		$WHYNOT='nav:nomessage';
		return 3;
	}

	if( !isset($MESSAGE) ) {

		$res = db_query( "SELECT * FROM ".$db_prefix."message "
			. "WHERE message_id=$msg_id" );
		if( !$res ) {
			$WHYNOT='nav:dberror';
			$msg_id=0;
			return 3;
		}
		if( db_numrows($res) < 1 ) {
			$WHYNOT='nav:invmessage';
			$msg_id=0;
			return 3;
		}
		$MESSAGE = db_fetch_array($res);
	}

	$forum_id = $MESSAGE['forum_id'];
	$thread_id = $MESSAGE['thread_id'];
	$linkadd = "?msg_id=$msg_id";
	return 0;
}


// thread -- validate the thread_id, then check the forum for
// private/public
function auth_thread() {
	global $WHYNOT, $linkadd, $thread_id;
	global $FORUM, $forum_id, $db_prefix;

	if( !isset($thread_id) || !$thread_id ) {
		$WHYNOT='nav:nothread';
		return 3;
	}

	// is it valid?
	$res = db_query( "SELECT * FROM ".$db_prefix."message "
		. "WHERE thread_id=$thread_id "
		. "AND followup_to=0" );
	if( !$res ) {
		$WHYNOT='nav:dberror';
		$thread_id=0;
		return 3;
	}
	if( db_numrows($res) < 1 ) {
		$WHYNOT='nav:invthread';
		$thread_id=0;
		return 3;
	}
	$THREAD = db_fetch_array($res);
	$forum_id=$THREAD['forum_id'];
	return 0;
}

?><?php
// For forum pages, a forum must be selected.  In case 'project' is also
// required, this sets the project_id.
function auth_forum($v) {
	global $WHYNOT, $linkadd, $forum_id;
	global $FORUM, $project_id, $auth_project, $db_prefix;

	if( !isset($forum_id) || !$forum_id ) {
		$WHYNOT='nav:noforum';
		return 3;
	}

	if( !isset($FORUM) ) {
		$res = db_query( "SELECT * FROM ".$db_prefix."forum "
			. "WHERE forum_id=$forum_id" );
		if( !$res ) {
			$WHYNOT='nav:dberror';
			$forum_id=0;
			return 3;
		}
		if( db_numrows($res) < 1 ) {
			$WHYNOT='nav:invforum';
			$forum_id=0;
			return 3;
		}
		$FORUM = db_fetch_array($res);
		$project_id = $FORUM['project_id'];
	}

	// private forum -- must be member of the respective project.
	if( !$FORUM['p_read'] ) $auth_project = 'm';

	if(!$linkadd) $linkadd = "?forum_id=$forum_id";
	return 0;
}

// projectcat edit - must be selected
function auth_projectcat() {
	global $WHYNOT, $linkadd, $projectcat_id;

	if( !isset($projectcat_id) || !$projectcat_id ) {
		$WHYNOT='nav:noprojectcat';
		return 3;
	}
	$linkadd = "?projectcat_id=$projectcat_id";
	return 0;
}

?><?php

// require that the session variable $reg_state be >= $v
function auth_reg_state($v) {
	global $WHYNOT, $reg_state;

	if( !isset($reg_state) || $reg_state < $v ) {
		$WHYNOT='nav:regstatelow';
		return 1;
	}

	return 0;
}

// can the current user edit the user "uid"?
function auth_editmember($u) {
	global $WHYNOT, $USER, $uid;

	if( !$u ) {
		if( !isset($uid) || !$uid ) {
			$WHYNOT='nav:nouser';
			return 1;
		}
		$u = $uid;
	}

	// validate this
	if( !user_org_perm('',$u) ) {
		$WHYNOT='nav:invuser';
		return 1;
	}

	// can edit self (but only if you're member)...
	if( isset($USER) && $USER['user_id'] == $u ) return 0; 
	// if admin of this org, OK
	if( user_org_perm('admin') ) return 0;

	// if general org admin, OK
	if( user_perm('orgmanage') ) return 0;

	// if allowed to chmod, and uid is *not* admin, OK
	if( user_org_perm('chmod') && !user_org_perm('admin',$u) ) return 0;

	$WHYNOT='nav:cantedit';
	return 1;
}

// can the user $USER edit the user $uid?
function auth_admin_edituser($u=0) {
	global $WHYNOT, $USER, $uid, $USR, $can_edit, $db_prefix;

	if( !isset($USER) )
		return 1;

	if( !$u ) {
		if( !isset($uid) || !$uid ) {
			$WHYNOT='nav:nouser';
			return 1;
		}
		$u = $uid;
	}

	if( !isset($USR) ) {
		$res = db_query("SELECT * FROM ".$db_prefix."user "
			. "WHERE user_id=$uid");
		if( !$res || db_numrows($res) < 1 ) {
			$WHYNOT='nav:invuser';
			return 1;
		}
		$USR = db_fetch_array($res);
	}

	$can_edit = 0;

	// you must either be site or user admin
	if( !$USER['p_useradmin'] && !$USER['p_siteadmin'] ) {
		return 0;
	}

	// The user $USER has the permission p_useradmin (sitemap).  But still
	// he can't edit site admins (except for himself).
	if( $USR['p_siteadmin'] && $USER['user_id'] != $USR['user_id'] ) {
		return 0;
	}

	$can_edit = 1;
	return 0;
}

?>
