<?php
/*
 *  Support for multiple languages.
 *  Copyright (C) 2000, 2001 Wolfgang Oertl.
 *
 *  Interface functions:
 *	gm($message_id,$vars,$langs,$flags); "get message"
 *	text_get($field_name,$object_id,[$language_code])
 *	lang_get_name($language_code)
 *	text_set
 *	text_del
 *
 * $Id: lang.php,v 1.14 2001/11/11 22:24:35 sf Exp $
 */

include "$INCDIR/vars.php";	// contains the FIELD_ID, LANG_IC, LANG_CI arrays.

// Default is all languages; it will be initialized from $USER[lang].
$LANG = array('en','de','es','fr');

// This array will be filled with all the messages loaded so far.
$MSG = array();

// Get the message with the ID $s in (if not otherwise specified) the first
// available language of the global array $LANG.
// If not found, returns the message key (which may be a normal text anyway).
// $flags: if 1, the [ed] link is not appended.
function gm($s, $vars='', $langs='', $flags=0) {
	global $MSG, $LANG, $MSG_LOADED, $message_dir;
	global $BASEDIR, $db_prefix;

	if( !is_array($vars) ) $vars = array();
	if( !is_array($langs) ) $langs = array();

	// is it a message id at all?
	$pos = strpos($s,':');
	$i = 0;
	if( !$pos || $pos > 15 || ($i=substr($s,$pos-1,1)=='\\') ) {
		if( $i ) $s = substr($s,0,$pos-1) . substr($s,$pos);
		return $s;
	}

	// maybe load file
	$modid = substr($s,0,$pos);
	if( !isset($MSG_LOADED[$modid]) ) {

		$fn = $BASEDIR.$message_dir.$modid.'.msg';
		error_log("Loading $message_dir$modid.msg");
		if( file_exists($fn) )
			include $fn;
		$MSG_LOADED[$modid] = 1;
	}

	if( !count($langs) )
		$langs = $LANG;

	// check $MSG
	$m = '';
	reset($langs);
	while( list($k,$lang)=each($langs) ) {
		if( isset($MSG[$s.'.'.$lang]) ) {
			$m = $MSG[$s.'.'.$lang];
			break;
		}
	}

	// If not found, try the DB (unless we already know it isn't there).
	if(!$m && !isset($MSG[$s.'.NO'])) {
		reset($langs);
		while(list($k,$lang)=each($langs)) {
			$res = db_query( "SELECT txt FROM ".$db_prefix."msg "
				."WHERE message_id='$s' AND language_code='$lang'");
			if( !$res ) { print db_query_error(); return $s; }
			if( db_numrows($res) > 0 ) {
				$m = db_result($res,0,'txt');
				break;
			}
		}

		// Store in MSG, so that another request returns the same
		// faster, or remember that this text doesn't exist.
		if($m)
			$MSG[$s.'.'.$langs[0]] = $m;
		else
			$MSG[$s.'.NO'] = 1;
	}

	// found?  If so, perform variable substitution
	if( $m ) {
		while( list($k,$v)=each($vars) )
			$m = str_replace("\$$k",$v,$m);

		// if it's not the first choice, offer to translate.
		if (!$flags && user_perm('siteadmin')) {
			if( $lang != $langs[0] ) {
				$m .= '<A href="'
					. get_link('account:trans_one')
					. '?message_id='
					. urlencode($s)
					. "&fromto=$lang:$langs[0]\">[tr]</A>";
			} else {
				$m .= '<A href="'
					. get_link('root:editmsg')
					. '?form_msgid='
					. urlencode($s)
					. "&form_lc=$lang\">[ed]</A>";
			}
		}

		return $m;
	}


	// Not found.  If it's a site admin, add a link to enter this message.
	// This results in invalid HTML code if the message is already
	// enclosed in <A>...</A>, but at least Netscape 4.7 renders it OK.
	if( user_perm('siteadmin') ) {
		$s .= '<A href="'
			. get_link('root:editmsg')
			. '?form_msgid='
			. urlencode($s)
			. '">[new]</A>';
	}

	return $s;
}

?><?php
// Get one or more strings from the database.  If a string isn't available in
// any of the languages given, return it in one of the languages it is
// available in.  If it isn't available at all, return NULL for these keys.
//
// field_names: array of field names to fetch, or single field name
// object_ids: array of object ids, or a single object id, or '' (all)
// language_codes: one or more languages, or '' for all
//
// Returns an array of (key, value) pairs:
//
//	key = $field_id : $object_id
//
//	Each of these IDs will be omitted if just one ID was queried for.
//	[If the key would be empty, just return the value directly (no
//	array). -- this is still disabled --wo]
//
//	$all == 0: pick the best, value = $text
//	$all == 1: pick the best, value = $lc:$text (CURRENTLY NOT IMPLEMENTED)
//	$all == 2: get all, key has $language_code appended, value=$text
//	$all == 3: get only texts languages in $language_codes.
//
function text_get($field_names, $object_ids, $language_codes='', $all=0) {
	global $LANG, $FIELD_ID, $db_prefix;
	$onefield = $oneobject = $onelang = 0;

	// check parameters
	if (!$field_names)
		return array();

	if (is_array($object_ids))
		if (!count($object_ids))
			return array();

	if (!is_array($field_names)) {
		$field_names = array($field_names);
		$onefield = 1;
	}

	if (!$language_codes)
		$language_codes = $LANG;

	// build a list of field_ids from the field names.
	$fields = array();
	reset($field_names);
	while (list($k,$fn) = each($field_names)) {
		$field_id = $FIELD_ID[$fn];
		if (!$field_id) {
			error_log("text_get(): Unknown field $fn",0);
			return array();
		}
		$fields[] = $field_id;
	}

	// select one or more fields
	if ($all == 2)
		$sql = "SELECT *";
	else
		$sql = "SELECT field_id,object_id,language_code,text_id";

	$sql .= " FROM $db_prefix"."text WHERE field_id";

	if (count($fields) == 1) {
		$sql .= "=$fields[0]";
	} else {
		$sql .= ' IN (' . join(',',$fields) . ')';
	}

	// select one or more objects
	if (is_array($object_ids)) {
		$sql .= ' AND object_id IN (' . join(',',$object_ids) . ')';
	} elseif ($object_ids) {
		$sql .= " AND object_id=$object_ids";
		$oneobject = 1;
	}

	$res = db_query($sql);
	if (!$res)
		print db_query_error();
	$out = array();

	// if all texts are to be returned, no more processing is required.
	if ($all == 2) {
		while ($row = db_fetch_array($res)) {
			$tmp = array();
			if (!$onefield)
				$tmp[] = $row['field_id'];
			if (!$oneobject)
				$tmp[] = $row['object_id'];
			if (!$onelang)
				$tmp[] = $row['language_code'];
			$key = join(':',$tmp);
			$out[$key] = $row['txt'];
		}
		return $out;
	}

	// build table of language priorities
	$lang_prio = array();
	for ($i=0; $i<count($language_codes); $i++)
		$lang_prio[$language_codes[$i]] = $i;

	// Find out which language version to use for which text.  The query
	// above always returns the texts in *all* languages - which allows to
	// fall back easily on the second-best etc. languages; otherwise,
	// multiple queries would be required.

	$tmp = array(); 	// key=[field_id][:][object_id]  value=text_id:language
	$keys = array();	// key=text_id	value=[field_id][:][object_id]
	while ($row = db_fetch_array($res)) {
		$key = $onefield ? '' : $row['field_id'];
		$key .= ($onefield || $oneobject) ? '' : ':';
		$key .= $oneobject ? '' : $row['object_id'];
		//error_log("row: key=$key, text_id=$row[text_id]");
		if (!isset($tmp[$key])) {
			$tmp[$key] = $row['text_id'] . ':' . $row['language_code'];
			$keys[$row['text_id']] = $key;
			continue;
		}

		// Is this language better?  Languages that are not in the
		// user's language list get priority 100 (very low).
		$lang_new = $row['language_code'];
		if ($all == 3 && !isset($lang_prio[$lang_new]))
			continue;

		list($textid_old, $lang_old) = split(':', $tmp[$key]);
		$prio_old = isset($lang_prio[$lang_old]) ? $lang_prio[$lang_old] : 100;
		$prio_new = isset($lang_prio[$lang_new]) ? $lang_prio[$lang_new] : 100;

		if ($prio_new < $prio_old) {
			unset($keys[$textid_old]);
			$tmp[$key] = $row['text_id'] . ':' . $lang_new;
			$keys[$row['text_id']] = $key;
		}
	}

	// Construct a list of texts that were not found; empty strings are
	// returned for them.
	// NOTE: I could just create empty strings for *all* requested keys
	// and just let them be replaced by the real thing later on.
	if (is_array($object_ids)) {
		$notfound = array();
		reset($object_ids);
		// key is [field_id:]object_id
		if ($onefield) {
			while (list($k, $key) = each($object_ids)) {
				if (!isset($tmp[$key]))
					$out[$key] = NULL;
			}
		} else {
			while (list($k, $ob_id) = each($object_ids)) {
				reset($fields);
				while (list($k, $f_id) = each($fields)) {
					$key = "$f_id:$ob_id";
					if (!isset($tmp[$key]))
						$out[$key] = NULL;
				}
			}
		}
		// print "NOTFOUND: " . join(',', array_keys($out)) . "<BR>\n";
	}



	// Are any left?
	if (count($tmp) == 0)
		return $out;

	// remove the language from the values
	reset($tmp);
	$tmp2 = array();
	while (list($k, $v) = each($tmp))
		$tmp2[] = intval( (substr($v,strpos(':',$v))) );
	sort($tmp2);

	// Get all the texts.
	$sql = "SELECT text_id,txt "
		. "FROM $db_prefix"."text "
		. "WHERE text_id IN (" . join(',',$tmp2) . ')';

	$res = db_query($sql);
	if (!$res)
		print db_query_error();
	while ($row = db_fetch_array($res)) {
		$key = $keys[$row['text_id']];
		$out[$key] = $row['txt'];
	}

	// Special case: just one object and one field in one language: return
	// just the string, no array.  Disabled for now.
//	if ($onefield && $oneobject && isset($out['']))
//		$out = $out[''];

	return $out;
}

?><?php

/*
 *  Add a text to the text table.  You have to make sure that you don't have
 *  any unquoted ' in $value.
 *  $language has to be a valid language code, e.g. `en', `de' etc.
 *
 *  RC: 0=ok  1=error
 */
function text_set($field_name,$object_id,$language_code,$value,$replace=0) {
	global $FIELD_ID, $db_prefix;

	if( !$object_id || !$language_code ) {
		error_log("text_set called w/o object or language.",0);
		return 1;
	}

	$field_id = $FIELD_ID[$field_name];
	if( !$field_id ) {
		error_log("field name $field_name not found.",0);
		return 1;
	}

	// maybe replace
	if( $replace ) {
		$res = db_query("UPDATE ".$db_prefix."text "
			. "SET txt='$value' "
			. "WHERE field_id=$field_id "
			. "AND object_id=$object_id "
			. "AND language_code='$language_code'");
		if( db_affected_rows($res) > 0 )
			return 0;
		// 17-10-2001
		// Unfortunately, when you UPDATE and nothing has changed,
		// db_affected_rows returns 0.	So, I have to find out
		// separately whether this entry already exists, if so,
		// return.
		$res = db_query("SELECT COUNT(*) AS cnt "
			. "FROM ".$db_prefix."text "
			. "WHERE field_id=$field_id "
			. "AND object_id=$object_id "
			. "AND language_code='$language_code'");
		if( !$res ) { print db_query_error(); return 1; }
		if(db_result($res,0,'cnt') > 0) { return 0; }
	}

	// Insert.  The attribute "text_id" is auto_increment!
	$res = db_query("INSERT INTO ".$db_prefix."text(field_id,object_id,language_code,txt) "
		."VALUES($field_id,$object_id,'$language_code','$value')");
	if( !$res )
		return 1;

	return 0;
}

// Remove text.  The language_code is optional.
function text_del($field_names,$object_id,$langs='') {
	global $FIELD_ID, $db_prefix;

	if( !$field_names || !$object_id )
		return;

	// just one parameter -> change into array
	if( !is_array($field_names) )
		$field_names = array($field_names);

	$ids = array();
	reset($field_names);
	while( list($k,$field_name) = each($field_names) ) {
		$field_id = $FIELD_ID[$field_name];
		if( !$field_id ) {
			error_log("field name $field_name not found.",0);
			return 1;
		}
		$ids[] = $field_id;
	}

	$cond = array();		// conditions
	if( count($ids) == 1 )
		$cond[] = "field_id=$ids[0]";
	else
		$cond[] = "field_id IN (" . join(',',$ids) . ")";

	if( is_array($langs) )
		$cond[] = 'language_code IN (' . join(',',$langs) . ')';
	elseif( $langs )
		$cond[] = "language_code='$langs'";

	$sql =	  "DELETE FROM ".$db_prefix."text "
		. "WHERE object_id=$object_id "
		. "AND " . join(" AND ",$cond);

	$res = db_query($sql);
	if( !$res ) {
		error_log("Deleting text failed: " . db_query_error(),0);
	}
}

?><?php

// returns an array: key=language_code, name=name.  Sorted by name.
// 17-06-2000	For languages without a name (none of the languages the user
//		selected), at least return the language code as name.
function lang_table() {
	global $LANG, $LANG_CI, $LANGTABLE;

	if( isset($LANGTABLE) )
		return $LANGTABLE;

	// key=language_id  value=name
	$names = text_get('language_name','',$LANG);
	$out=array();
	while(list($lc,$id)=each($LANG_CI)) {
		$s = isset($names[$id]) ? $names[$id] : $lc;
		$out[$lc] = $s;
	}
	asort($out);
	$LANGTABLE = $out;
	return $out;
}


function lang_get_name($language_code) {
	global $LANG, $LANG_CI;

	// find the language's ID
	$id = $LANG_CI[$language_code];
	if( !$id ) return '';

	// get the language's best name.
	$names = text_get( 'language_name', $id, $LANG );
	return isset($names['']) ? $names[''] : $language_code;
}

?><?php

// allow the user to change the language setting anytime.
function lang_footer() {
	global $USER, $LANG, $REQUEST_URI;

	// not logged in -- just inform
	if( !isset($USER) ) {
		reset($LANG);
		print "Content will be displayed in:<B>";
		while(list($k,$v)=each($LANG)) {
			print " $v";
		}
		print "</B>. Log in to get more options.\n";
		help('lang:notloggedin');
		return;
	}

	reset($LANG);
	list($k,$v) = each($LANG);
	print "<B>$v</B>";

	// The new links have the same GET arguments as the current page,
	// but with "xlc" added/replaced.  Nothing can be done about POST
	// stuff.
	$s = ereg_replace('xlc=..&?','',$REQUEST_URI);
	if( substr($s,-1,1) != '&' ) {
		$s .= strpos($s,'?') ? '&' : '?';
	}

	// each language should be a link that switches to this lang.
	while(list($k,$v)=each($LANG))
		print ' <A href="'.$s."xlc=$v\">$v</A>\n";
	help('lang:footer');
	// print "<br>\n";
}

// The user clicks on a language link at the bottom.  Update the user table
// and $LANG: the selected language moves in front, the rest is unchanged.
function lang_change() {
	global $LANG, $USER, $xlc, $db_prefix;

	if( !isset($xlc) || !$xlc || !isset($USER) ) return;
	$newlang = array($xlc);
	reset($LANG);
	while(list($k,$v)=each($LANG))
		if( $v != $xlc ) $newlang[] = $v;
	$LANG = $newlang;

	db_query( "UPDATE ".$db_prefix."user "
		. "SET languages='".join(' ',$LANG)."' "
		. "WHERE user_id=$USER[user_id]" );
}

?>
