<?php
/* This module handles the retrieval of session data as well as storing such
 * data.  Creation of new sessions is handled in newsession.php.
 *
 * Global variables:
 *	$USER		result from SELECT * FROM user WHERE ...
 *	$USER_VAR	contains user variables (from $USER[vars])
 *	$SESSION	result from SELECT * FROM session WHERE ...
 *	$SESSION_VAR	contains session variables (see session_var_sync)
 *	$MODE		browser mode. N/T/F/S.  This is == $USER[browser_mode].
 *	$AUTHFAIL	contains the reason why a user wasn't authenticated.
 *
 * $Id: session.php,v 1.8 2001/11/11 21:45:32 sf Exp $
 */

$SESSION_VAR = array();
$MODE = 'N';			// N=normal  T=textonly  F=frames  S=simple

function session_cookie($name,$value) {
	error_log("Sending a cookie of name $name and value $value");
	setcookie($name,$value,0,'/','',0);
}

// verify session_hash & set up the session.  RC: 0=ok 1=err 2=dberr
// On error $AUTHFAIL will contain an explanation of what went wrong.
function session_set_try()
{
	global $session_hash, $USER, $SESSION, $AUTHFAIL, $db_prefix;

	if( !isset($session_hash) || !$session_hash ) {
		$AUTHFAIL='Cookie "session_hash" not set.';
		return 0;
	}

	$AUTHFAIL='';

	$res = db_query("SELECT * FROM ".$db_prefix."session "
		. "WHERE session_hash='$session_hash'");
	if( !$res ) {
		$AUTHFAIL='Database failure: ' . db_query_error();
		return 2;
	}
	$sess = db_fetch_array($res);
	if( !$sess ) {
		$AUTHFAIL='The session_hash you provided does not exist.';
		error_log("session $session_hash not found.",0);
		return 1;
	}

	// verify the IP address.
	if( $GLOBALS['REMOTE_ADDR'] != $sess['ip_addr'] ) {
		error_log("IP addr mismatch.",0);
		$AUTHFAIL='Your IP address has changed, which is not permitted.  Log in again.';
		return 1;
	}

	// OK.  Refresh the "when_last" timestamp when it's already a bit old
	// (keepalive), but disallow really old timestamps (timeout).
	$now = time();
	global $session_do_timeout, $session_timeout, $session_keepalive;
	if( $session_do_timeout ) {
		if( $sess['when_last'] + $session_timeout < $now ) {
			error_log("Timeout.",0);
			$AUTHFAIL='Your session has timed out.';
			return 1;
		}
		if( $sess['when_last'] + $session_keepalive < $now ) {
			error_log("Keepalive.",0);
			db_query("UPDATE $db_prefix"."session "
				. "SET when_last=$now "
				. "WHERE session_hash='$session_hash'");
			$sess['when_last'] = $now;
		}
	}

	// get the user's info.
	$res = db_query("SELECT * FROM ".$db_prefix."user "
		. "WHERE user_id=$sess[user_id]");
	if( !$res ) return 2;
	$USER = db_fetch_array($res);

	// User doesn't exist -> invalid session, delete it.
	if( !isset($USER) || !$USER ) {
		error_log("Invalid session $session_hash for user $sess[user_id].",0);
		$AUTHFAIL='Your session is there, but your user doesn\'t exist...';
		return 1;
	}

	// The user must be active (not pending/deleted/suspended)
	if( $USER['status'] != 'A' ) {
		$AUTHFAIL=gm("session:user_$USER[status]");
		return 1;
	}

	// set the languages.
	if( $USER['languages'] ) {
		global $LANG;
		$LANG = explode(' ',$USER['languages']);
	}

	global $MODE;
	$MODE = $USER['browser_mode'];

	// Maybe the user wants to switch the language.  In this case a GET
	// argument "xlc=<lang>" is set.
	lang_change();

	// All OK.
	$SESSION = $sess;

	// now get the user and session variables
	global $USER_VAR, $SESSION_VAR;
	$USER_VAR = unserialize($USER['vars']);
	$SESSION_VAR = unserialize($SESSION['vars']);
	if(!is_array($SESSION_VAR))
		$SESSION_VAR = array();
	// extracts into the name space of this function, not very useful...
	// extract($SESSION_VAR);
	while(list($k,$v) = each($SESSION_VAR)) {
		# print "setting global var $k to $v<br>\n";
		$GLOBALS[$k] = $v;
	}

	return 0;
}

// wrapper that disposes of the session when it's invalid.
function session_set() {
	global $session_hash, $db_prefix;

	$rc = session_set_try();
	if( $rc != 1 ) return $rc;

	db_query("DELETE FROM ".$db_prefix."session "
		. "WHERE session_hash='$session_hash'");
	session_cookie('session_hash','');
	return 1;
}


function session_issecure() {
	return (getenv('SERVER_PORT') == '443');
}

function session_redirect($loc) {
	header('Location: http' . (session_issecure()?'s':'') . '://' . getenv('HTTP_HOST') . $loc);
	print("\n\n");
	exit;
}

?><?php

// ------------------- Session/User variables --------------------- 

// 05-08-2000: instead of manually serializing all variables in SESSION_VAR
// and using 'parse_str' to unserialize, the functions 'serialize' and
// 'unserialize' are now used.
//
// To create & initialize variables: call {session,user}_var_add()
// To remove variables: call {session,user}_var_del()
// To use/change variables: simply use the global variable.
//


// Register a new session/user variable and initialize it (unless it already
// exists).
function session_var_add($name, $default='') {
	global $SESSION_VAR;

	if (!isset($SESSION_VAR[$name])) {
		$SESSION_VAR[$name] = $default;
		$GLOBALS[$name] = $default;
	}
}

function user_var_add($name, $default='') {
	global $USER_VAR;

	if (!isset($USER_VAR[$name])) {
		$USER_VAR[$name] = $default;
		$GLOBALS[$name] = $default;
	}
}

// Set a session/user variable
function session_var_set($name, $value) {
	global $SESSION_VAR;
	$SESSION_VAR[$name] = $value;
}

function user_var_set($name, $value) {
	global $USER_VAR;
	$USER_VAR[$name] = $value;
}


// Unset one or more session/user variables
function session_var_del($names) {
	global $SESSION_VAR;

	if (!is_array($names))
		$names=array($names);

	while (list($key,$name) = each($names)) {
		unset($SESSION_VAR[$name]);
	}
}

function user_var_del($names) {
	global $USER_VAR;

	if (!is_array($names))
		$names=array($names);

	while (list($key,$name) = each($names)) {
		unset($USER_VAR[$name]);
	}
}


// If any changes have been made to the session/user variables, write them
// into the DB.
function session_var_sync() {
	global $SESSION, $SESSION_VAR, $db_prefix;

	if (!isset($SESSION))
		return;

	// gather new values
	reset ($SESSION_VAR);
	while (list($key, $val) = each($SESSION_VAR)) {
		if (isset($GLOBALS[$key]))
			$SESSION_VAR[$key] = $GLOBALS[$key];
		# else print "Warning: variable $key vanished.";
		# print "current value of $key is: $SESSION_VAR[$key]<br>\n"; # vs. $GLOBALS[$key]\n";
	}

	// Store if changed.
	$s = serialize($SESSION_VAR);
	if ($s == $SESSION['vars'])
		return;

	$res = db_query("UPDATE $db_prefix"."session "
		. "SET vars='$s' "
		. "WHERE session_hash='$SESSION[session_hash]'");
	if ($res)
		$SESSION['vars'] = $s;
}

function user_var_sync() {
	global $USER, $USER_VAR, $db_prefix;

	if (!isset($USER))
		return;

	// gather new values
	reset ($USER_VAR);
	while (list($key, $val) = each($USER_VAR)) {
		if (isset($GLOBALS[$key]))
			$USER_VAR[$key] = $GLOBALS[$key];
	}

	// Store if changed.
	$s = serialize($USER_VAR);
	if ($s == $USER['vars'])
		return;

	$res = db_query("UPDATE $db_prefix"."user "
		. "SET vars='$s' "
		. "WHERE user_id=$USER[user_id]");
	if ($res)
		$USER['vars'] = $s;
}

?>
