<?php
// vim:fo=:

/* This is the Site Map.  There's one entry per page; key=pagename,
 * value=array:
 *
 *	parent		the higher level page in the heirarchy.
 *	link		how to reach it
 *	dontshow	don't show this page in "nav_next".
 *	other:		a permission check; see auth_xxx functions.
 *
 * Note: order is important - that's the order the entries will show up in
 * nav_next().
 */

/*

In preparation to split this file analogous to the messages, adopt following
pagename scheme: <domain>:<page>.

Whenever a page should appear in a page's "nav_next" list, it must be in the
same domain.  Example:

	The "search for projects" page appears on the "Root" page, and
	therefore should be in this domain (root:plist) and not in the
	project domain (project:list), because then nav_next would not show
	it.

*/

/*
 *  For each entry a function with the name auth_xxx is called.  If the
 *  function doesn't exist, or if it returns non-zero, the authentication
 *  fails.  Exception: link and parent, which are not checked.
 */

$SITEMAP = array(
	'Root' => array('link'=>'/'),

	// login/new user
	'login' => array('parent'=>'Root','link'=>'/account/login.php','logout'=>1),
	'pending' => array('parent'=>'login','logout'=>1),
	'user_register' => array('parent'=>'login','link'=>'/account/register.php','logout'=>1),
	'verify' => array('parent'=>'login','link'=>'/account/verify.php','logout'=>1),

	'root:search' => array('parent'=>'Root','link'=>'/project/search.php'),

	// site administration
	'admin' => array('parent'=>'Root','link'=>'/admin/','superuser'=>1),
	'admin:userlist' => array('parent'=>'admin','link'=>'/admin/userlist.php'),
	'admin:projectlist' => array('parent'=>'admin','link'=>'/admin/projectlist.php'),
	'admin:edituser' => array('parent'=>'admin:userlist','link'=>'/admin/edituser.php','login'=>1,'admin_edituser'=>''),
	'admin:lastlogins' => array('parent'=>'admin','link'=>'/admin/lastlogins.php','superuser'=>1),
	'catlist' => array('parent'=>'admin','link'=>'/admin/catlist.php','superuser'=>1),
	'catedit' => array('parent'=>'catlist','link'=>'/admin/catedit.php','superuser'=>1,'projectcat'=>1),
	'catdel' => array('parent'=>'catedit','link'=>'/admin/catdel.php','superuser'=>1,'projectcat'=>1),
	'catadd' => array('parent'=>'catlist','link'=>'/admin/catedit.php','superuser'=>1),
	'langedit' => array('parent'=>'admin','link'=>'/admin/langedit.php','superuser'=>1),
	'langedit_lang' => array('parent'=>'langedit','superuser'=>1),
	'langimport' => array('parent'=>'admin','link'=>'/admin/langimport.php','superuser'=>1),
	'langexport' => array('parent'=>'admin','link'=>'/admin/langexport.php','superuser'=>1),
	'langbuildresult' => array('parent'=>'langimport','superuser'=>1),
	'langbuild' => array('parent'=>'admin','link'=>'/admin/langbuild.php','superuser'=>1),
	'admin:projectpend' => array('parent'=>'admin','link'=>'/admin/pending-projects.php','user'=>'projectmanage'),
	'admin:editoct' => array('parent'=>'admin','link'=>'/admin/editoct.php','superuser'=>1),
	'root:editmsg' => array('parent'=>'Root','link'=>'/misc/editmsg.php','superuser'=>1),

	'root:importer' => array('parent'=>'Root','link'=>'/misc/importer.php','superuser'=>1),

	// account maintenance
	'my' => array('parent'=>'Root','link'=>'/my/','login'=>1),
	'acc_maint' => array('parent'=>'my','link'=>'/account/','login'=>1),
	'change_lang' => array('parent'=>'acc_maint','link'=>'/account/change_lang.php','login'=>1),
	'change_pw' => array('parent'=>'acc_maint','link'=>'/account/change_pw.php','login'=>1),
	'change_realname' => array('parent'=>'acc_maint','link'=>'/account/change_realname.php','login'=>1),
	'first_login' => array('parent'=>'my','login'=>1,'link'=>'/account/first.php','dontshow'=>1),

	'account:translator' => array('parent'=>'my','link'=>'/account/translator.php','login'=>1,'user'=>'translate'),
	'account:translate' => array('parent'=>'my','link'=>'/account/translate.php','login'=>1,'user'=>'translate'),
	'account:trans_one' => array('parent'=>'account:translate','link'=>'/account/t_one.php','dontshow'=>1,'login'=>1,'user'=>'translate'),

	// projects
	'root:plist' => array('parent'=>'Root','link'=>'/project/list.php'),
	'project:main' => array('parent'=>'Root','link'=>'/project/','project'=>''),
	'project:edit' => array('parent'=>'project:main','link'=>'/project/editproject.php','project'=>'A'),
	'project:addlang' => array('parent'=>'project:edit','project'=>'A'),
	'project:members' => array('parent'=>'project:main','link'=>'/project/members.php','project'=>''),
	'project:adduser' => array('parent'=>'project:members','link'=>'/project/adduser.php','project'=>'adduser'),
	'project:edituser' => array('parent'=>'project:members','link'=>'/project/edituser.php','project'=>'','projectedituser'=>''),
	'project:cat' => array('parent'=>'project:main','link'=>'/project/editcat.php','project'=>'A'),
	'project:addorg' => array('parent'=>'project:members','link'=>'/project/addorg.php','project'=>'adduser'),

	// forums
	'project:forums' => array('parent'=>'project:main','link'=>'/forum/','project'=>''),
	'forum:add' => array('parent'=>'project:forums','link'=>'/forum/addforum.php','project'=>'F'),
	'forum:forum' => array('parent'=>'project:forums','link'=>'/forum/forum.php','forum'=>'','project'=>''),
	'forum:del' => array('parent'=>'forum:forum','link'=>'/forum/delforum.php','forum'=>'','project'=>'F'),
	'forum:thread' => array('parent'=>'forum:forum','link'=>'/forum/thread.php','thread'=>1,'forum'=>'','project'=>''),
	'forum:message' => array('parent'=>'forum:thread','link'=>'/forum/message.php','message'=>1,'forum'=>'','project'=>''), 	// need PROJECT[status]
	'forum:delmsg' => array('parent'=>'forum:message','link'=>'/forum/delmsg.php','message'=>1,'forum'=>'','project'=>'F'),

	// organisations
	'org' => array('parent'=>'Root','link'=>'/org/'),
	'addorg' => array('parent'=>'org','link'=>'/org/addorg.php','login'=>1),
	'org:view' => array('parent'=>'org','link'=>'/org/vieworg.php','org_id'=>1),
	'editorg' => array('parent'=>'org:view','link'=>'/org/editorg.php','org_id'=>1,'org_perm'=>'chfn'),
	'editmember' => array('parent'=>'org:view','org_id'=>1,'org_perm'=>'','editmember'=>''),
	'delorg' => array('parent'=>'org:view','link'=>'/org/delorg.php','org_id'=>1,'org_status'=>'AP','org_perm'=>'admin'),
	'purgeorg' => array('parent'=>'org:view','link'=>'/org/purgeorg.php','org_id'=>1,'org_status'=>'D'),
	'delorgresult' => array('parent'=>'org'),
	'activateorg' => array('parent'=>'org:view','link'=>'/org/activateorg.php','org_id'=>1,'org_status'=>'DP'),
	'org_adduser' => array('parent'=>'org:view','link'=>'/org/adduser.php','org_id'=>1,'org_status'=>'A','org_perm'=>'adduser'),
	'org:join' => array('parent'=>'org:view','link'=>'/org/join.php','org_id'=>1,'org_status'=>'A','org_perm'=>'admin'),
	'org:editcontact' => array('parent'=>'org:view','link'=>'/org/editcontact.php','org_id'=>1,'org_status'=>'A','org_perm'=>'chfn'),

	// registration
	'root:register' => array('parent'=>'Root','link'=>'/register/','login'=>1),
	'register:requirements' => array('parent'=>'root:register','link'=>'/register/requirements.php','login'=>1),
	'register:tos' => array('parent'=>'register:requirements','link'=>'/register/tos.php','login'=>1),
	'register:basicinfo' => array('parent'=>'register:tos','link'=>'/register/basicinfo.php','login'=>1),
	'register:projectname' => array('parent'=>'register:basicinfo','link'=>'/register/projectname.php','login'=>1,'reg_state'=>1),
	'register:category' => array('parent'=>'register:projectname','link'=>'/register/category.php','login'=>1,'reg_state'=>2),
	'register:confirmation' => array('parent'=>'register:category','link'=>'/register/confirmation.php','login'=>1,'reg_state'=>3),

	'root:sitemap' => array('parent'=>'Root','link'=>'/misc/sitemap.php'),
	'logout' => array('parent'=>'Root','link'=>'/account/logout.php','login'=>1),

	// no special permissions required to see *this* :-(
	'error' => array('parent'=>'Root','link'=>'/misc/error.php','dontshow'=>1),
	'root:nopage' => array('parent'=>'Root','link'=>'/misc/nopage.php','dontshow'=>1),
	'help' => array('parent'=>'foo','link'=>'/misc/help.php'),
	'source' => array('parent'=>'Root','link'=>'/misc/source.php','dontshow'=>1)
);

?>
